package PureMessage::Manager::CgiLogger; 


require Exporter; 
our @ISA = qw(Exporter);
our @EXPORT = qw(); 
our @EXPORT_OK = qw(t d);
our $VERSION = '1.0';

use FileHandle;
use Data::Dumper;
use PureMessage::Manager::Config qw(%web_config);
use strict;


=pod

=head1 NAME

PureMessage::Manager::CgiLogger - Perl extension for writing trace messages to a commong log file used in trouble shooting

=head1 SYNOPSIS

 use PureMessage::Manager::CgiLogger qw(t d);
 t 'got to here';
 t 'value of $a is ' . d($a);
 {
 	local $PureMessage::Manager:CgiLogger::On = 0;
 	t 'this message will not be printed';
 }

 $PureMessage::Manager::CgiLogger::Base = '/opt/pmx/var/log/mylogdir';
 
 $PureMessage::Manager::CgiLogger::LogFile = 'myownfile.log';

 $PureMessage::Manager::CgiLogger::Append = 1;
 # logger will now open the log file in append mode (default is to overwrite the file for each transaction)


=head1 DESCRIPTION

This module is an easy and consistant way for PureMessage Manager CGI scripts to write status messages to a common CGI log file. 

Much of the API and code was borrowed from the CPAN module L<Log::TraceMessages>. This implementation removes some unneeded functionality and adds 
some specialization for particluar use in this application (i.e. only outputs to a file, standardized the location of files).

=head1 USAGE

=over

=item $PureMessage::Manager::CgiLogger::On

Flag controlling whether tracing is on or off.  You can set it as you
wish, and of course it can be C<local>-ized.  The default is on.

=cut
our $On = 1;

=pod


=item $PureMessage::Manager::CgiLogger::LogFile

The name of the file to which trace should be appended. By default this is set to:
I<manager.log>. You can override this to produce your own log file. This is useful for
scripts that create forked processes and you don't want the forked process to overwrite
the CGI trace.

All log files will be created in the standard logging directory, specified by I<$web_config{'log_dir'}> unless I<Base> is set manually.

=cut
our $Logfile = 'manager.log';
my $curr_Logfile = $Logfile;
my $fh = undef;

=pod

=item $PureMessage::Manager::CgiLogger::Base

The base directory that the log file will be created in. 

=cut
our $Base = $web_config{'log_dir'};

=pod

=item $PureMessage::Manager::CgiLogger::Append

A flag to turn append mode on or off (default). If this is turned on before the first call to C<t()>
then the log file will be opend in append mode otherwise the log file is overwritten.

=cut
our $Append = 0;

=pod

=item t(messages)

Print the given strings, if tracing is enabled. Newlines will be automatically appended 
to each string if needed.

=cut


sub t(@) {
	return unless $On;
	
	if (defined $Logfile) {
		unless (defined $curr_Logfile and $curr_Logfile eq $Logfile) {
			if (defined $fh) {
				close $fh;
			}
			undef $fh;
		}
		if (not defined $fh) {
			my $base = $Base;
			unless($base =~ /\/$/){
				$base .= '/';
			}
			$base .= $Logfile;

			if($Append){
				$fh = new FileHandle(">>$base") or die "cannot append to $base: $!";
			}else{
				$fh = new FileHandle(">$base") or die "cannot open $base: $!";
			}
			$fh->autoflush(1);
			$curr_Logfile = $Logfile;
		}

		my $s;
		foreach $s (@_) {
			my $a = "\n" unless $s =~ /\n\z/;
			print $fh "$s$a" or die "cannot print to filehandle: $!";
		}
	}
}

=pod

=item d(scalar)

Return a string representation of a scalar's value suitable for use in
a trace statement.  This is just a wrapper for Data::Dumper.

C<d()> will exit with '' if trace is not turned on.  This is to
stop your program being slowed down by generating lots of strings for
trace statements that are never printed.

=back

=cut
sub d($) {
	return '' if not $On;
	require Data::Dumper;
	my $s = $_[0];
	my $d = Data::Dumper::Dumper($s);
	$d =~ s/^\$VAR1 =\s*//;
	$d =~ s/;$//;
	chomp $d;
	return $d;
}

1;
__END__

