#!perl

package PureMessage::Manager::Response::Support::Snapshots;

use strict;

use PureMessage::Manager::Response::Support;
use PureMessage::Manager::Widget qw(button);
use PureMessage::Manager::Lang qw(text);
use PureMessage::Manager::Widget::ToolBar;
use PureMessage::Manager::Widget::Grid;
use PureMessage::Manager::Config qw(%web_config);
use PureMessage::Manager::Utils qw(urlize html_escape);
use PureMessage::Snapshot;
use PureMessage::Service;
use PureMessage::Version qw($PREFIX);
use Date::Parse qw(str2time);

use base qw(PureMessage::Manager::Response::Support);

return 1;

sub list_snapshots {
    my $self = shift;
    my @grid = ();

    if(my(@snapshots) = PureMessage::Snapshot->snapshots) {
        foreach my $i (
            sort { str2time($a->{date}) <=> str2time($b->{date}) }
            @snapshots
        ) {
            push(@grid, [
                button({
                    type => "text", text => html_escape($i->{name}),
                    href => "?view=" . $i->{dir_name}
                }),
                $i->{date}, $i->{desc}
            ]);
        }
    }
    
    $self->addGrid(
        title   => text("current_snapshots"),
        columns => 3,
        data    => \@grid,
        width   => q{100%},
        ddata   => [
            { description => text("name"), ddata => "nowrap=nowrap" },
            { description => text("date"), ddata => "nowrap=nowrap" },
            { description => text("desc"), ddata => "nowrap=nowrap" }
        ]
    );

    $self->print(q{<form method="get">});
            
    $self->addGrid(
        title   => text('create_new_snapshot'),
        columns => 2,
        data    => [
            [
                text("name_p"),
                q{<INPUT TYPE="TEXT" NAME="create" /> } .
                q{<INPUT TYPE="SUBMIT" VALUE="} . text('create') . q{" />}
            ], [
                text("desc_p"),
                q{<INPUT TYPE="TEXT" NAME="desc" />}
            ]
        ],
        width   => q{100%}
    );
     
    $self->print("</form>");
            
    return 1;
}

sub create_snapshot {
    my($self, $snap_name, $snap_desc) = @_;
    my($err, $snapshot);
    
    eval {
        $snapshot = PureMessage::Snapshot->new(
            name        => $snap_name,
            desc        => $snap_desc,
            create      => 1,
            quiet       => 1
        );
        1;
    } or $err = $@;
    
    if($err) {
        $self->print(
            "<P><B>", text('snapshot_creation_failed'), "</B></P>",
            "<P><EM>$err</EM></P>",
            "<P>", text('manager_error_log'), "</P>"
        );
        return $self->list_snapshots;
    } else {
        $self->print(
            "<P><B>", text('new_snapshot_created'), "</B></P>"
        );
        return $self->view_snapshot($snapshot->{_dir_name});
    }
}

sub snapshot_by_dir_name {
    my($self, $dir_name, $noraise) = @_;
    if(my($snapshot) =
        (grep($_->{dir_name} eq $dir_name, PureMessage::Snapshot->snapshots))
    ) {
        # in array context, return the most commonly desired manipulations
        if(wantarray) {
            return(
                $snapshot, $self->snapshot_grid($snapshot),
                html_escape($snapshot->{dir_name})
            );
        } else {
            return $snapshot;
        }
    } else {
		$self->raise_error(text("snapshot_not_found"))
			unless($noraise);
        return;
	}
}

sub view_snapshot {
    my($self, $dir_name) = @_;
    
    if(my($snapshot, $grid, $e_dir) = ($self->snapshot_by_dir_name($dir_name))) {
        my $toolbar = PureMessage::Manager::Widget::ToolBar->new(
            buttons => [[
                { type => "text", href => "?restore=$e_dir",
                    text => text("restore_this_snapshot") },
                { type => "text", href => "?delete=$e_dir",
                    text => text("delete_this_snapshot") },                        
            ]]
        );
                    
        $self->print($grid->output, $toolbar->output);
            
        return 1;
    } else {
        return;
    }
}

sub snapshot_grid {
    my($self, $snapshot) = @_;

    my @grid = ();
    
    foreach my $attr (qw(name date desc diff flavour)) {
        if($snapshot->{$attr} && ref($snapshot->{$attr}) eq "ARRAY") {
            push(@grid, [
                "<b>" . text("snapshot_attr_$attr") . "</b>",
                "<ul><li><pre>" .
                    join(
                        "</pre></li><li><pre>",
                        map { html_escape($_) } @{$snapshot->{$attr}}
                    ) .
                    "</pre></li></ul>"
            ]);
        } elsif($snapshot->{$attr}) {
            push(@grid, [
                "<b>" . text("snapshot_attr_$attr") . "</b>",
                "<pre>" . html_escape($snapshot->{$attr}) . "</pre>"
            ]);
        }
    }
    
    my $grid = PureMessage::Manager::Widget::Grid->new(
        title   => text('snapshot_label', $snapshot->{name}),
        columns => 2,
        width   => "100\%",
        data    => \@grid
    );
        
    $grid;
}

sub confirm_snapshot_action {
    my($self, $action, $dir_name) = @_;
    
    if(my($snapshot, $grid, $e_dir) = ($self->snapshot_by_dir_name($dir_name))) {
        my $toolbar = PureMessage::Manager::Widget::ToolBar->new(buttons => [[
            { type => "text", href => "?confirm_$action=$e_dir",
				text => text("confirm_$action") } ,
            { type => "text", href => "?view=$e_dir",
                text => text("abort_$action") }
        ]]);
        
        $self->print(
            "<p><b>", text('please_confirm_p'), "</b></p>",
            $toolbar->output, $grid->output
        );
        
        return 1;
    } else {
        return;
    }
}

sub delete_snapshot {
    my($self, $dir_name) = @_;
    
    if(my($snapshot, $grid) = ($self->snapshot_by_dir_name($dir_name))) {
        my $snapshot_obj = PureMessage::Snapshot->new(
            name         => $snapshot->{name},
            _dir_name    => $snapshot->{dir_name}
        );
        if(eval { $snapshot_obj->delete; 1; }) {
            $self->print(
			    "<p><b>", text('delete_success'), "</b></p>", $grid->output
            );
            
            return 1;
        } else {
            $self->print("<p><b>", text("delete_failure", $@), "</b></p>");
            
            return $self->view_snapshot($dir_name);
        }
    } else {
        return;
    }
}

sub restore_snapshot {
    my($self, $dir_name) = @_;
    
    if(my($snapshot, $grid) = ($self->snapshot_by_dir_name($dir_name))) {
        my $httpd = PureMessage::Service->new('http');
        my $e_dir = $dir_name;
		my $refself = ref($self);
        $e_dir =~ s{\"}{\\\"}g;

        if(my $error = $httpd->soft_restart(
			qq{use $refself; ${refself}\->do_restore_snapshot("$e_dir");\n}
        )) {
            $self->print(
                "<p><b>", text('error_restarting_manager', $error), "</b></p>"
            );
            
            return 0;
        } else {
            $self->add_head(
                q{<META HTTP-EQUIV="Refresh" CONTENT="5; ?finish_restore=} .
                html_escape(urlize($e_dir)) . q{" />}
            );
            
            $self->print(
                "<P><B>", text("restarting_p"), "</B></P>",
                "<P>", text("be_patient", "?finish_restore=$e_dir"), "</P>"
            );
            
            return 1;
        }
    } else {
        return;
    }
}

sub finish_restore_snapshot {
    my($self, $dir_name) = @_;
    
    if(my($snapshot, $grid, $e_dir) = ($self->snapshot_by_dir_name($dir_name))) {
        my $toolbar = PureMessage::Manager::Widget::ToolBar->new(
            buttons => [[
                { type => "text", href => "?delete=$e_dir",
                    text => text("delete_this_snapshot") },                        
            ]]
        );
        
        my $log;
                    
        if(-e($self->ppm_log_file)) {
            $log = $self->get_ppm_log;
            unlink($self->ppm_log_file);
        } else {
            $log = text("log_file_missing", $self->ppm_log_file);
        }
                    
        $self->print(
            "<p><b>", text('manager_restarted'), "</b></p>",
            "<p>", text('results_of_restore_p'), "</p>",
            "<PRE>$log</PRE>",
            $grid->output,
            $toolbar->output
        );

        return 1;
    } else {
        return;
    }
}

# $self passed here may be a class, not just a ref.
# snapshot_by_dir_name and SUPER::write_ppm_log have to be safe with
# this operation.
sub do_restore_snapshot {
    my($self, $dir_name) = @_;

    if(my $snapshot = $self->snapshot_by_dir_name($dir_name)) {
       my $snapshot_obj = PureMessage::Snapshot->new(
           name         => $snapshot->{name},
           _dir_name    => $snapshot->{dir_name}
       );
		   
	   system("$PREFIX/bin/pmx", "stop");
		   
       if(eval { $snapshot_obj->restore(1); 1; }) {
           $self->write_ppm_log(text('restore_successful', $dir_name));
		   system("$PREFIX/bin/pmx", "start");
           return 1;
       } else {
           $self->write_ppm_log(text('restore_failed', $dir_name, $@));
		   system("$PREFIX/bin/pmx", "start");
           return 0;
       }
    }        
}
