package PureMessage::Manager::Response::Support::Update;
use strict;
use ActiveState::Install;
use PureMessage::Manager::Request;
use PureMessage::Manager::Response::Support;
use PureMessage::Manager::Lang;
use PureMessage::Service;
use Exporter;
use PPM qw(ListOfRepositories QueryInstalledPackages VerifyPackage
           InstalledPackageProperties InstallPackage RemovePackage
           RepositoryPackageProperties RemoveRepository
          );

use PureMessage::Manager::Config qw(%config %web_config);
use PureMessage::Version qw($PREFIX);


our @ISA = ('PureMessage::Manager::Response::Support','Exporter');
our @EXPORT_OK = qw(versionToDots);


# returns a hash that contains only the installed PureMessage packages
# the returned structure will be a hash keyed on the package name.
#$VAR0 = 'PureMessage-KeywordCheck';
#$VAR1 = {
#           'DATE' => 'Wed Sep 18 10:47:08 2002',
#           'ABSTRACT' => 'Look for keywords in the message and take various actions based on hits',
#           'NAME' => 'PureMessage-KeywordCheck',
#           'PERLCORE_VER' => undef,
#           'DEPEND' => [
#                         'PureMessage-Milter-Scanner',
#                         'Config-General'
#                       ],
#           'AUTHOR' => 'ActiveState Corp',
#           'TITLE' => 'PureMessage-KeywordCheck',
#           'VERSION' => '2,0,0,200'

sub pmxInstallList{
	my %summary = InstalledPackageProperties();
	foreach(keys %summary) {
		delete $summary{$_} unless /PureMessage/;
	}
	return %summary;
}

sub get_pkg_version{
	my $pkg = shift;
	my %pkgs = pmxInstallList();
	return $pkgs{$pkg}->{'VERSION'};
}

sub pkgNeedsUpdate{
        my $this = shift;
	my $location = shift;
	my $pkgName = shift;
	
	my $rc = VerifyPackage('location'=>$location, 'package'=>$pkgName, 'upgrade'=>0);
	
	return $rc;
}

# This funciton always succeeds. The return value will be the new package's
# version number. This function will croak if there was a failure.
sub upgrade_package{
	my ($repository, $package) = @_;
	
	my $rc = VerifyPackage('location'=>$repository, 'package'=>$package, 'upgrade'=>1);

	# PPM returns 0 if the package was up to date, 1 if an upgrade
	# is available (never in this case) or undef if there was an error.
	# If the package was actually upgraded, the new version is returned.
	if(! defined($rc)){
                # TODO xxx we have now webmin_log just yet
		#webmin_log('upgrade', 'package', $package, {'repository'=>$repository, 'errmsg'=>$PPM::PPMERR});
		die $PPM::PPMERR;
	}
	
	# We always gaurantee that the new package's version is returned.
	# If the package was already up to date then return the installed package's
	# version.
	if($rc == 0){
		$rc = get_pkg_version($package);
	}
	PureMessage::Config::validate_cache();
	#webmin_log('upgrade', 'package', $package, {'repository'=>$repository, 'version'=>$rc});
	
	return $rc;	
}

# This function returns the new package's version number.
# If an error occurs it will croak.
sub install_package{
	my ($repository, $package) = @_;

	my $rc = 0;
	if($repository){
		$rc = InstallPackage('location'=>$repository, 'package'=>$package);
		#webmin_log('install', 'package', $package, {'repository'=>$repository, 'success'=>$PPM::ERR});
		die $PPM::PPMERR unless $rc;
	}else{
		$rc = InstallPackage('package'=>$package);
		#webmin_log('install', 'package', $package, {'repository'=>'DEFAULT', 'success'=>$PPM::ERR});
		die $PPM::PPMERR unless $rc;
	}
	PureMessage::Config::validate_cache() if $rc;

	return get_pkg_version($package);	
}

sub remove_package{
        my $this = shift;
	my $package = $_[0];
	my $rc = RemovePackage('package'=>$package);
	#webmin_log('remove', 'package', $package, {'status'=>$rc});
	PureMessage::Config::validate_cache() if $rc;
	# print "Remove result is: $rc<br>\n";
	return $rc;
}

sub needUpdateList{
        my $this = shift;
	my $repository = $_[0];	
	
	# get the list of packages that need to be updated
	my %pkgs = pmxInstallList();
	my %updates;
	foreach my $pkgname(keys(%pkgs)){
		my $rc = $this->pkgNeedsUpdate($repository, $pkgname);	
                
                # save the status regardless, if not found it will be defined,
                # or a boolean indicating if it should be updated
	        $updates{$pkgname} = $rc;
	}
	return {'updates'=>\%updates, 'pkgs'=>\%pkgs};
}

# getAvailablePackages()
# returns a hash (not a reference) that contains the package name
# for each package that is available to be installed
# and weather the package is actually installed. (and the package description)
# i.e.
# return {PureMessage-VirusCheck=>[0,'description'] PureMessage-SpamCheck=>[1, description]}
# where 0 == available and 1 == installed
#
sub getAvailablePackages{
        my $this = shift;
	my @pkgs = $this->getLicencedPackages();
	my %installed = pmxInstallList();
	my %available;
	foreach my $pkg(@pkgs){
		if($installed{$pkg}){
			$available{$pkg} = [1,$this->getPkgInfo($pkg, \%installed)->{'ABSTRACT'}];
		}else{
			$available{$pkg} = [0,$this->getPkgInfo($pkg, \%installed)->{'ABSTRACT'}];
		}
	}
	return %available;
}
sub getLicencedPackages{
        my $this= shift;
	my @pkgs = ();

	require PureMessage::_sys;
	my %lic =  PureMessage::_sys::lic_features(PureMessage::_sys::lic_file());
	
	# inlcude only licensed packages for PureMessage 
	foreach(keys(%lic)){
		s/::/-/g;
		push (@pkgs, $_) if(/PureMessage/);
	}
	#print "Licensed packages: @pkgs<br>\n";
	return @pkgs;
}

sub hide_as_repository{
        my $this = shift;
	# don't set save as we want don't want to permanently
	# alter this repository - only for this session.
	RemoveRepository('repository'=>'ActiveState Package Repository');
}

# transparently gets information about a package locally or from the PPM repository
sub getPkgInfo{
        my $this = shift;
	my $name = shift;
	my $installed = shift;
	return $$installed{$name} if $$installed{$name};
	my %info = RepositoryPackageProperties('package'=>$name);
	return \%info;
	
}

# converts a version string from comma format to dot format
# i.e. 2,0,0,200 becomes 2.0.0.200
#
sub versionToDots{
	my $ver = shift;
	$ver =~ s/,/\./g;
	return $ver;
}


# start and end tracking functions disply simple output that
# records the progress of installations and upgrades. 
sub start_tracking{
        my $this = shift;
	my $type = shift;
	my $package = shift;
	my $version = get_pkg_version($package);
	
	$this->print( text('start_upgrade_tracking', $package, versionToDots($version)))
                if $type eq 'upgrade';
	$this->print( text('start_install_tracking', $package))
                if $type eq 'install';
	
	return {'package'=>$package, 'version'=>$version, 'type'=>$type};
}

sub end_tracking{
        my $this = shift;
	my $opts = shift;
	my $status_img;
	
	# Now that the package should have been installed,
	# display the package installation/upgrade information.
	if($$opts{'status'}){
		$this->print(text('end_upgrade_tracking_ok', $$opts{'package'},
                                      versionToDots($$opts{'version'}), 
                                      versionToDots($$opts{'new_version'})
                                      )
                                ) if $$opts{'type'} eq 'upgrade';

		$this->print(text('end_install_tracking_ok', $$opts{'package'},
                                  versionToDots( $$opts{'new_version'} ) ) ) if $$opts{'type'} eq 'install';
	}else{
		$this->print(text('end_upgrade_tracking_fail', $$opts{'package'}))
                        if $$opts{'type'} eq 'upgrade';
		$this->print(text('end_install_tracking_fail', $$opts{'package'}))
                        if $$opts{'type'} eq 'install';
	}

}

# Display each package and its status as it is upgraded or installed.
sub display_install{
        my  $this = shift;
	my $type = shift;
	my $repo = shift;
	my @pkgs = @_;
        my @pid;      
        my @services = PureMessage::Service->Available;
        my $RESTORE_SIG = $SIG{'TERM'};
        $SIG{'TERM'} = 'IGNORE'; # stop process from being killed by restart
        
        my @running;
        foreach my $service ( @services) {
          push(@running,$service) if $service->type ne 'http' and $service->running();
        }
      
	my ($pkgs_inst, $pkgs_failed, $pkgs_ok) = (0,0,0);
	
        eval{
            push(@pid,$_->stop( bg=> 1))foreach @running;
            waitpid($_,0) foreach @pid; # wait for them all to shutdown
            # write_ppm_log will clear log file for us
            $this->write_ppm_log(text('update_service_stop_text'),1);
            foreach(@pkgs){
                    my $tracking = $this->start_tracking($type, $_);
                    
                    # call the ppm command and record any output.
                    eval{
			    $this->record_ppm_action($type, $repo, $_);
                    };
                    if($@){
                            $pkgs_failed++;
                            $$tracking{'status'} = 0;
                            # should also log this error some place (the ppm log?)
                    }else{
			    my $pkg = ActiveState::Install::installed (( etc => "$PREFIX/etc",
									 pkg => $_ ));
                            $pkgs_ok++;
                            $$tracking{'status'} = 1;
                            $$tracking{'new_version'} = $pkg->{'version'};
                    }
                    $pkgs_inst++;
                    $this->end_tracking($tracking);
                    
            }
            undef(@pid);
            push(@pid,$_->start(bg=>1)) foreach @running;
            # if you don't wait on each started process manager soft_restart wont' work
            waitpid($_,0) foreach @pid;
            my $httpd = PureMessage::Service->new('http');
            $this->write_ppm_log(text('update_service_start_text') . "\n");
            if (my $error = $httpd->soft_restart() ) {
                $this->raise_error("Failed to restart Manager Service: $error");
                $this->write_ppm_log("Failed to restart Manager Service: $error");
            }
            else {
                $this->write_ppm_log(text('update_service_manager_restart_text') . "\n");
            }
            
            $this->display_tracking_summary($type, $pkgs_inst, $pkgs_ok, $pkgs_failed);
	};

	if($@){
		$this->print("FAILED: $@");
	}
}

# display a summary of the packages that were tracked during
# installation/upgrade and provide the user with a link to the
# log.
sub display_tracking_summary{
        my $this = shift;
	my $type = shift;
	my $npackages = shift;
	my $nok = shift;
	my $nfailed = shift;
	my $WIDGET = new PureMessage::Manager::Widget;
	$this->print($WIDGET->notification_header_str());
	$this->print(text('upgrade_tracking_summary', $npackages, $nok, $nfailed)) if $type eq 'upgrade';
	$this->print(text('upgrade_tracking_message', $npackages, $nok, $nfailed)) if $type eq 'upgrade';
	$this->print(text('install_tracking_summary', $npackages, $nok, $nfailed)) if $type eq 'install';
	$this->print(text('install_tracking_message', $npackages, $nok, $nfailed)) if $type eq 'install';
	$this->print($WIDGET->notification_footer_str());
}

# log the ppm output for future use
sub record_ppm_action{
        my $this = shift;
	my $action = shift;
	my $location = shift;
	my $pkg = shift;
	my $log_file = $this->ppm_log_file;
        my $log_dir = substr($log_file, 0, rindex($log_file, "/"));

	# ?? how can we be running if this isn't here ... ?
	if( ! -d $log_dir){
		require File::Path;
		File::Path::mkpath($log_dir) or
			die "Failed to create the directory $log_dir. $!";
	}

	# Build up the arguments to be passed to ppm for the action
	my $action_args;
	if ($action eq 'upgrade') {
	    $action_args = "verify --upgrade";
	} elsif ($action eq 'install') {
	    $action_args = "install";
	} else {
	    die "Unknown action, $action\n";
	}

	# call the ppm command
        my $cmd = "ppm $action_args --location $location $pkg 1>>$log_file 2>&1";
	system ($cmd) == 0
	    or die "$cmd failed, status $?\n";
}

1;
