package PureMessage::Manager::Setup;
use strict;
use warnings;
use PureMessage::Manager::Config qw(%web_config $conf_error save_web_config reload_web_config);
use PureMessage::Manager::Users qw(:all $users_err_str);
use File::Find;
use Socket;

require Exporter; 
our @ISA = qw(Exporter);
our @EXPORT_OK = qw(
	verify_setup
	db_ok
	db_perms_ok
	fix_db_perms
	mk_empty_db
	mk_groupdb
	has_old_dbs
	upgrade_old_dbs
	get_uid_rules
	update_cgis
	update_libs
	update_files
	generate_config
	change_port
	manager_connect_test
);
our %EXPORT_TAGS = ('all'=>\@EXPORT_OK);

# return a false value if any test fails
sub verify_setup{
	my %options = @_;
	my $report = $options{'report'} || undef;
	my $ok = 1;

	# check each db file
	my $userdb_mode = $options{'userdb-mode'} || '0';
	my @dbs = qw(user acl group membership);
	foreach my $db (@dbs){
		print "Checking DB file $db" if($report);
		my $rc = db_ok($db);
		if( ! $rc){
			print " Failed\n" if($report);
			$ok = 0;
			fix_db_perms($db, oct($userdb_mode)) unless($report);
		}
		else{
			print " OK.\n" if($report);
		}
	}
	
	# check the permissions on each .cgi and .pl file
	my $cgi_mode = $options{'cgi-mode'} || '0';
	$ok = 0 unless(update_cgis('report'=>$report, 'mode'=>oct($cgi_mode)));
	
	my $pl_mode = $options{'pl-mode'} || 0;
	$ok = 0 unless(update_libs('report'=>$report, 'mode'=>oct($pl_mode)));
	
	return $ok;
}

sub db_ok{
	my $db = shift;
	return (-e $web_config{'user_acl_db'}) if($db eq 'acl');
	return (-e $web_config{'group_db'}) if($db eq 'group');
	return (-e $web_config{'membership_db'}) if($db eq 'membership');
	if($db eq 'user'){
		return 0 unless(-e $web_config{'user_db'});
		
		# an empty user DB is considered missing too
		open_db();
		my %users = list_users();
		return 0 unless %users;
	}
	return 1;	
}

sub mk_empty_db{
	my $db = shift;
	my $mode = shift || 0600;
	my $file = _get_file($db);
	
	# backup any old db file then generate a 0 byte, empty db file.
	# XX what if $file.bak already exists?
	rename $file, "$file.bak" if -e $file;
	open(FH, ">$file") or die "Failed to create the db file $file. $!";
	close FH;
	
	# not atomic but will override the umask which will ensure
	# we get the permissions that were specified.
	chmod $mode, $file; 
	return 1;
}

# Adds the default admin group to a new group database
sub mk_groupdb{
	my $mode = shift || 0;
	
	# create the empty db
	mk_empty_db('group', $mode);
	
	# add the group
	open_db();
	add_group('admin');
	grant_group_acl('admin', ['*']);
	return commit_db();
}

sub has_old_dbs{
	return 0;
}

sub upgrade_old_dbs{
	my $mode = shift || 0600;
	return 1;
}

sub get_uid_rules{
	my $rules = <<EOT;
User names can only be made up of the following characters:
    a-z, A-Z, 0-9, _, -, !, .
    
And be between 1 character and 20 characters long.
EOT
	return $rules;
}

sub db_perms_ok{
	my $db = shift;
	my $mode = shift || 0600;
	my $file = _get_file($db);
	
	my $st = (stat($file))[2];
	return( ($st & 07777) == $mode);
}	

sub fix_db_perms{
	my $db = shift;
	my $mode = shift || 0600;
	my $file = _get_file($db);

	return chmod $mode, $file;
}

# returns true value if they don't match (the file's permissions)
sub bad_file_perms{
	my $file = shift;
	my $type = shift;
	my $mode = shift || _get_file_mode($type);
	my $st = (stat($file))[2];
	$st = ($st & 07777);
	$st = 0 if($st == $mode);
	return $st;
}

# Check each directory for each installed package and update
# any CGI's that have the wrong permissions. Don't do any
# updates if in report mode.
sub update_cgis{
	my %opts = @_;
	$opts{'pattern'} = '\.cgi$';
	$opts{'type'} = 'cgi';
	return update_files(%opts);
}
# Do the same for .pl files
sub update_libs{
	my %opts = @_;
	$opts{'pattern'} = '\.pl$';
	$opts{'type'} = 'pl';
	return update_files(%opts);
}

# update .cgi or .pl file permissions
sub update_files{
	my %opts = @_;
	
	my $mode = $opts{'mode'} || _get_file_mode($opts{'type'});
	my $pat = $opts{'pattern'};
	
	open_db();
	my $pkgs = sys_get_available_packages();
	my $base  = $web_config{'root'};
	my @dirs;
	foreach(keys %$pkgs ){
		push(@dirs, "$base/$_") if(-d "$base/$_");
	}
	
	# code ref that is applied to each file in the search directories
	my $wanted = sub{
		
		# XX should also check to make sure we are only modifying files
		# that are truly in the manager root.
		#
		return unless /$pat/;
		
		# special cases (don't update these this way)
		return if /miniserv.pl/;
		
		# get the permissions of the file if they are not right then fix it
		print "Check file $File::Find::name\n" if($opts{'report'});
		my $perms = bad_file_perms($File::Find::name, $opts{'type'}, $mode);
		if($perms){
			if($opts{'report'}){
				printf("File %s has incorrect permissions. Has %lo, expected %lo\n",
					$File::Find::name, $perms, $mode);
			}else{
				chmod($mode, $File::Find::name);
			}
		}
	};
	my $find_ops = {'wanted'=>$wanted, 'no_chdir'=>1};
	find($find_ops, @dirs);
}


sub generate_config{
	my %opts = @_;
	
	# back up any existing one if force is turned on
	if($opts{'force'} && -e $web_config{'conf_file'}){
		# XX what if there already is a .bak?
		rename $web_config{'conf_file'}, "$web_config{'conf_file'}.bak";
	}

	# save the auto-generated defaults; reset to defaults if force is on.
	reload_web_config() if($opts{'force'});
	save_web_config(%web_config);
	
}

sub _get_file_mode{
	my $type = shift;
	return 0440 if($type eq 'pl');
	return 0550 if($type eq 'cgi');
}

sub _get_file{
	my $db = shift;
	my $file;
	
	$file = $web_config{'user_db'} if($db eq 'user');
	$file = $web_config{'user_acl_db'} if($db eq 'acl');
	$file = $web_config{'membership_db'} if($db eq 'membership');
	$file = $web_config{'group_db'} if($db eq 'group');
	return $file;
}

sub change_port{
	my $port = shift;
	my $force = shift || 0;

	# do nothig if the port is already the port used by the Manager
	return 1 if($port == $web_config{'port'});

	unless($force){

		# Try 3 times to bind the port.
		socket(S, PF_INET, SOCK_STREAM, getprotobyname('tcp')) ||
			die "Failed to open socket : $!";
		setsockopt(S, SOL_SOCKET, SO_REUSEADDR, pack("l", 1));
		my $i;
		for($i = 0; $i < 3; $i++) {
			last if (bind(S, sockaddr_in($port, INADDR_ANY)));
			sleep(1);
		}
		close S;
		if($i == 5){
			# If the port is busy but it is the Manager listening then it's ok
			return 0 unless manager_connect_test($port);
		}
	}
	
	# save the changed port
	$web_config{'port'} = $port;
	save_web_config( %web_config );
	return 1;	
}

# XX Should be part of the Manager::Network lib
sub manager_connect_test{
	my $port = shift;
	use LWP::UserAgent;
	my $ua = new LWP::UserAgent; 
	my $r = '';
	my $h = 'localhost';
	
	# connect to the web server and read its response headers
	eval{
		$r = $ua->get("http://$h:$port");
	};
	return 0 if $@;
	
	# the web server should have returned a 200 status code and
	# "PureMessage"in the Server: header. This should work even
	# if it is running in SSL mode.
	return 0 unless $r;
	return 0 unless($r->code == 200);
	return 0 unless($r->server =~ /PureMessage/);
	
	return 1;
}



1;
